@@ -1,52 +1,10 @@
 import requests
 import json
-import re
-from datetime import datetime
-from janda.utils import parser, just_number
+from janda.utils.parser import *
 
-BASE_URL = 'https://nhentai.sinxdr.workers.dev'
-BASE_IMG = 'https://i.nhentai.net/galleries'
-
-class Nhentai(object):
-
-    @staticmethod
-    def preg_match_tags(tags):
-        """Parses the tags and returns a list of tags.
-
-        Parameters
-        ----------
-        tags : str
-            The tags to be parsed.
-        
-        Returns
-        -------
-        list
-        """
-
-        tags_name = []
-        for tag in tags:
-            tags_name.append(tag['name'])
-        return tags_name
-
-    @staticmethod
-    def get_language_in_tags(tags):
-        """Parses language value
-
-        Parameters
-        ----------
-        tags : list
-            The tags to be parsed.
-
-        Returns
-        -------
-        str
-        """
-
-        for tag in tags:
-            if tag['type'] == 'language':
-                return tag['name']
-    
+BASE_URL = Api()
 
+class Nhentai(object):    
 
     def __init__(self, api_key: str = ''):
         """Initializes the client.
@@ -54,7 +12,7 @@ class Nhentai(object):
         Parameters
         ----------
         api_key : str
-            scathach.dev API key (optional)
+            scathach.dev API key for pururin (optional)
         
         Returns
         -------
@@ -66,7 +24,7 @@ class Nhentai(object):
             self.api_key = api_key
         self.specs = {'api_key': self.api_key}
 
-    async def get_book(self, book: int, safe_search: bool = None):
+    async def get(self, book: int, safe: bool = None):
         """Get doujin API from Id
 
         Parameters
@@ -74,7 +32,7 @@ class Nhentai(object):
         book : int
             Nmber id of the book
 
-        safe_search : bool
+        safe : bool
             If True, janda will throw you error whenever contains minor content, such as loli or shota. Default is False
 
         Raises
@@ -88,23 +46,31 @@ class Nhentai(object):
         """
 
         try:
+
             if isinstance(book, int):
                 book = str(book)
 
             self.book = str(just_number(book))
 
         except ValueError:
-            raise ValueError('Book must be an int')
+            raise ValueError('Book must be contain int')
 
-        data = requests.get(F'{BASE_URL}/api/gallery/{self.book}')
+        data = requests.get(F'{BASE_URL.api}/api/gallery/{self.book}')
         if data.status_code == 404:
             raise ValueError('Book not found')
 
         if data.status_code != 200:
             raise ValueError('Request failed')
 
+        self.tags = preg_match_tags(data.json()['tags'])
+
+        if safe is True:
+            if 'lolicon' in self.tags or 'shotacon' in self.tags:
+                raise ValueError('Book contains minor content')
+        
         return parser(data.json())
 
+
     async def search(self, tags: str, page: int = 1, popular: str = 'today'):
         """Search doujin by tags / artis / character / parody or group
 
@@ -137,15 +103,16 @@ class Nhentai(object):
         self.specs['popular'] = popular
 
         data = requests.get(
-            f'{BASE_URL}/api/galleries/search', params=self.specs)
+            f'{BASE_URL.api}/api/galleries/search', params=self.specs)
+
+        if data.status_code == 404:
+            raise ValueError('No results')
 
         if data.status_code != 200:
             raise ValueError('Request failed')
 
-        if data.status_code != 200:
-            raise ValueError('Request failed')
 
-        self.raw_object = json.loads(Nhentai.better_object(data.json()))
+        self.raw_object = json.loads(better_object(data.json()), encoding="utf-8")
         self.results = self.raw_object['result']
 
         self.results_object = []
@@ -154,16 +121,17 @@ class Nhentai(object):
                 'id': result['id'],
                 'title': result['title'],
                 'link': f'https://nhentai.net/g/{result["id"]}',
-                'upload_date': Nhentai.readable_timestamp(result['upload_date']),
+                'upload_date': readable_timestamp(result['upload_date']),
                 'num_pages': result['num_pages'],
                 'num_favorites': result['num_favorites'],
-                'language': Nhentai.get_language_in_tags(result['tags']),
-                'tags': Nhentai.preg_match_tags(result['tags'])
+                'language': get_language_in_tags(result['tags']),
+                'tags': preg_match_tags(result['tags'])
             })
 
-        return Nhentai.better_object(self.results_object)
+        return better_object(self.results_object)
 
-    async def get_related(self, book: int):
+
+    async def search_related(self, book: int):
         """Get realated book API from book ID or book link
 
         Parameters
@@ -182,38 +150,22 @@ class Nhentai(object):
         """
 
         try:
-            self.book = str(Nhentai.just_number(book))
+            if isinstance(book, int):
+                book = str(book)
+
+            self.book = str(just_number(book))
 
         except ValueError:
             raise ValueError('Book must be an int')
 
-        data = requests.get(F'{BASE_URL}/api/gallery/{self.book}/related')
+        data = requests.get(F'{BASE_URL.api}/api/gallery/{self.book}/related')
 
         if data.status_code != 200:
             raise ValueError('Request failed')
 
-        if data.status_code != 200:
-            raise ValueError('Request failed')
+        return neat_result(data.json())
 
-        self.raw_object = json.loads(Nhentai.better_object(data.json()))
-        self.results = self.raw_object['result']
-
-        self.results_object = []
-        for result in self.results:
-            self.results_object.append({
-                'id': result['id'],
-                'title': result['title'],
-                'link': f'https://nhentai.net/g/{result["id"]}',
-                'upload_date': Nhentai.readable_timestamp(result['upload_date']),
-                'num_pages': result['num_pages'],
-                'num_favorites': result['num_favorites'],
-                'language': Nhentai.get_language_in_tags(result['tags']),
-                'tags': Nhentai.preg_match_tags(result['tags'])
-            })
-
-        return Nhentai.better_object(self.results_object)
-
-    async def random(self):
+    async def get_random(self):
         """Get random doujin
         
         Raises
@@ -225,19 +177,20 @@ class Nhentai(object):
         -------
         dict
         """
-        random = requests.get(F'{BASE_URL}/random')
+        random = requests.get(F'{BASE_URL.api}/random')
 
         try:
-            self.book = str(Nhentai.just_number(random.url))
+            self.book = str(just_number(random.url))
 
         except ValueError:
             raise ValueError('Uh oh something wrong here')
 
-        data = requests.get(F'{BASE_URL}/api/gallery/{self.book}')
+        data = requests.get(F'{BASE_URL.api}/api/gallery/{self.book}')
+        
         if data.status_code == 404:
             raise ValueError('Error')
 
         if data.status_code != 200:
             raise ValueError('Request failed')
 
-        return Nhentai.parser(data.json())
+        return parser(data.json())
